"use strict";
/*
This file is part of uvcc -- USB Video Class (UVC) device configurator.
Copyright (C) 2018, 2019, 2020, 2021 Joel Purra <https://joelpurra.com/>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const command_1 = require("./types/command");
class CommandManager {
    constructor(output, cameraFactory, cameraHelperFactory, commandHandlers) {
        Object.defineProperty(this, "output", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: output
        });
        Object.defineProperty(this, "cameraFactory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: cameraFactory
        });
        Object.defineProperty(this, "cameraHelperFactory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: cameraHelperFactory
        });
        Object.defineProperty(this, "commandHandlers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: commandHandlers
        });
        assert_1.default.strictEqual(arguments.length, 4);
        assert_1.default(typeof this.output === "object");
        assert_1.default(typeof this.cameraFactory === "object");
        assert_1.default(typeof this.cameraHelperFactory === "object");
        assert_1.default(typeof this.commandHandlers === "object");
    }
    async execute(runtimeConfig) {
        this.output.verbose("Parsed arguments:", JSON.stringify(runtimeConfig, null, 2));
        const commandName = runtimeConfig.cmd;
        const commandHandlerExists = await this.commandHandlers.has(commandName);
        if (!commandHandlerExists) {
            this.output.error("Unknown command:", commandName);
            return;
        }
        let camera = null;
        const closeCamera = async () => {
            if (camera) {
                await camera.close();
            }
        };
        const commandArguments = await this.commandHandlers.getArguments(commandName);
        // TODO: smarter solution.
        if (commandArguments.includes(command_1.CommandHandlerArgumentCameraHelper)) {
            const { vendor, product, address, } = runtimeConfig;
            assert_1.default(typeof vendor === "number");
            assert_1.default(typeof product === "number");
            assert_1.default(typeof address === "number");
            camera = await this.cameraFactory.get(vendor, product, address);
        }
        try {
            // NOTE HACK: don't mix "injection" with other arguments.
            const argumentValues = commandArguments
                .filter((commandArgument) => commandArgument !== command_1.CommandHandlerArgumentCameraHelper)
                .map((commandArgument) => runtimeConfig[commandArgument]);
            // TODO: smarter solution.
            if (commandArguments.includes(command_1.CommandHandlerArgumentCameraHelper)) {
                assert_1.default(camera !== null);
                const cameraHelper = await this.cameraHelperFactory.get(camera);
                argumentValues.unshift(cameraHelper);
            }
            const output = await this.commandHandlers.execute(commandName, ...argumentValues);
            // NOTE: could separate of types of commands -- those with output, and those without.
            if (typeof output !== "undefined") {
                const json = JSON.stringify(output, null, 2);
                this.output.normal(json);
            }
            await closeCamera();
        }
        catch (error) {
            try {
                await closeCamera();
            }
            catch (_a) {
                // NOTE: if there were previous errors, ignore errors when closing the camera connection.
            }
            throw error;
        }
    }
}
exports.default = CommandManager;
