#!/usr/bin/env node
"use strict";
/*
This file is part of uvcc -- USB Video Class (UVC) device configurator.
Copyright (C) 2018, 2019, 2020, 2021 Joel Purra <https://joelpurra.com/>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const engine_check_1 = __importDefault(require("engine-check"));
const uvc_control_1 = __importDefault(require("uvc-control"));
const camera_control_helper_1 = __importDefault(require("./camera-control-helper"));
const camera_control_helper_factory_1 = __importDefault(require("./camera-control-helper-factory"));
const camera_factory_1 = __importDefault(require("./camera-factory"));
const camera_helper_1 = __importDefault(require("./camera-helper"));
const camera_helper_factory_1 = __importDefault(require("./camera-helper-factory"));
const command_handlers_1 = __importDefault(require("./command-handlers"));
const controls_1 = __importDefault(require("./command-handlers/controls"));
const devices_1 = __importDefault(require("./command-handlers/devices"));
const export_1 = __importDefault(require("./command-handlers/export"));
const get_1 = __importDefault(require("./command-handlers/get"));
const import_1 = __importDefault(require("./command-handlers/import"));
const range_1 = __importDefault(require("./command-handlers/range"));
const ranges_1 = __importDefault(require("./command-handlers/ranges"));
const set_1 = __importDefault(require("./command-handlers/set"));
const command_manager_1 = __importDefault(require("./command-manager"));
const output_1 = __importDefault(require("./output"));
const runtime_configurator_1 = __importDefault(require("./runtime-configurator"));
const uvc_device_lister_1 = __importDefault(require("./uvc-device-lister"));
const mainAsync = async () => {
    try {
        // NOTE: ignoring unhandled rejections and exceptions, as there is (practically) nothing to gracefully shut down.
        const runtimeConfig = runtime_configurator_1.default();
        const output = new output_1.default(runtimeConfig.verbose);
        process.on("unhandledRejection", (...args) => {
            output.error(...args);
            process.exitCode = 1;
        });
        process.on("uncaughtException", (...args) => {
            output.error(...args);
            process.exitCode = 1;
        });
        const cameraFactory = new camera_factory_1.default(output, uvc_control_1.default);
        const cameraControlHelperFactory = new camera_control_helper_factory_1.default(uvc_control_1.default, camera_control_helper_1.default);
        const cameraHelperFactory = new camera_helper_factory_1.default(output, cameraControlHelperFactory, camera_helper_1.default);
        const uvcDeviceLister = new uvc_device_lister_1.default(uvc_control_1.default);
        const commands = {
            controls: new controls_1.default(),
            devices: new devices_1.default(uvcDeviceLister),
            "export": new export_1.default(),
            get: new get_1.default(),
            "import": new import_1.default(),
            range: new range_1.default(),
            ranges: new ranges_1.default(),
            set: new set_1.default(),
        };
        const commandHandlers = new command_handlers_1.default(output, commands);
        const commandManager = new command_manager_1.default(output, cameraFactory, cameraHelperFactory, commandHandlers);
        await commandManager.execute(runtimeConfig);
    }
    catch (error) {
        // NOTE: root error handler for asynchronous errors.
        // eslint-disable-next-line no-console
        console.error(error);
        process.exitCode = 1;
    }
};
const main = () => {
    try {
        engine_check_1.default();
        void mainAsync();
    }
    catch (error) {
        // NOTE: root error handler for synchronous errors.
        // eslint-disable-next-line no-console
        console.error(error);
        process.exitCode = 1;
    }
};
main();
