"use strict";
/*
This file is part of uvcc -- USB Video Class (UVC) device configurator.
Copyright (C) 2018, 2019, 2020, 2021 Joel Purra <https://joelpurra.com/>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <https://www.gnu.org/licenses/>.
*/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const chalk_1 = require("chalk");
const find_up_1 = __importDefault(require("find-up"));
const yargs_1 = __importDefault(require("yargs"));
const getJsonSync = (fileRelativePath) => {
    const resolvedPath = path_1.join(__dirname, fileRelativePath);
    try {
        // eslint-disable-next-line no-sync
        const json = JSON.parse(fs_1.default.readFileSync(resolvedPath).toString());
        return json;
    }
    catch (error) {
        throw new Error(`Could not read JSON file ${JSON.stringify(resolvedPath)}: ${JSON.stringify(String(error))}`);
    }
};
const getYargsArgv = () => {
    const packageJson = getJsonSync("../package.json");
    const appBinaryName = Object.keys(packageJson.bin)[0];
    const appDescription = packageJson.description;
    const { homepage, } = packageJson;
    assert_1.default(typeof appBinaryName === "string");
    assert_1.default(typeof homepage === "string");
    const epilogue = chalk_1.dim `uvcc Copyright © 2018, 2019, 2020, 2021 Joel Purra <https://joelpurra.com/>\n\nThis program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under certain conditions. See GPL-3.0 license for details.\n\nSee also: ${homepage}`;
    let fromImplicitConfigFile = null;
    const hasConfigFlag = process.argv.includes("--config");
    if (hasConfigFlag) {
        fromImplicitConfigFile = {};
    }
    else {
        const nearestConfigPath = find_up_1.default.sync([
            `.${appBinaryName}rc`,
            `.${appBinaryName}rc.json`,
        ]);
        const configFromNearestConfigPath = nearestConfigPath ? getJsonSync(nearestConfigPath) : {};
        fromImplicitConfigFile = configFromNearestConfigPath;
    }
    /* eslint-disable @typescript-eslint/prefer-readonly-parameter-types */
    yargs_1.default
        .strict()
        .wrap(yargs_1.default.terminalWidth())
        .config(fromImplicitConfigFile)
        .config("config", "Load command arguments from a JSON file.", (argumentConfigPath) => {
        const fromExplicitConfigFile = argumentConfigPath ? getJsonSync(argumentConfigPath) : {};
        return fromExplicitConfigFile;
    })
        .env(appBinaryName.toUpperCase())
        .usage(`${chalk_1.bold("$0")}: ${appDescription}`)
        .command("get <control>", "Get current control value.", (yargsToApplyTo) => {
        yargsToApplyTo
            .positional("control", {
            demandOption: true,
            describe: "Name of the control.",
            type: "string",
        });
    })
        .command("set <control> <value1> [value2]", "Set control value(s).", (yargsToApplyTo) => {
        yargsToApplyTo
            .positional("control", {
            demandOption: true,
            describe: "Name of the control.",
            type: "string",
        })
            .positional("value1", {
            demandOption: true,
            describe: "Value to set.",
            type: "number",
        })
            .positional("value2", {
            demandOption: false,
            describe: "Second value to set, if supported by the control.",
            type: "number",
        });
    })
        .command("range <control>", "Get possible range (min and max) for a control.", (yargsToApplyTo) => {
        yargsToApplyTo
            .positional("control", {
            demandOption: true,
            describe: "Name of the control.",
            type: "string",
        });
    })
        .command("ranges", "Get all ranges (min and max).")
        .command("devices", "List connected UVC devices.")
        .command("controls", "List all supported controls for the camera.")
        .command("export", "Output configuration in JSON format, on stdout.")
        .command("import", "Input configuration in JSON format, from stdin.")
        .option("vendor", {
        "default": 0,
        describe: "Camera vendor id (vId).",
        type: "number",
    })
        .option("product", {
        "default": 0,
        describe: "Camera product id (pId).",
        type: "number",
    })
        .option("address", {
        "default": 0,
        describe: "Camera device address.",
        type: "number",
    })
        .option("verbose", {
        "default": false,
        describe: "Enable verbose output.",
        type: "boolean",
    })
        .demandCommand(1, 1, chalk_1.red("Please provide a single command."), chalk_1.red("Please provide a single command."))
        .group([
        "vendor",
        "product",
        "address",
    ], chalk_1.bold("Device selection for multi-camera setups.") + "\n  " + chalk_1.dim("Numbers in hex (0x000) or decimal (0000) format."))
        .help()
        .example("", "")
        .example(chalk_1.bold("Basic usage:"), "")
        .example("$0 controls", "Available controls for the camera.")
        .example("$0 set auto_white_balance_temperature 0", "Turn off automatic color correction.")
        .example("$0 set saturation 64", "Low color saturation (near grayscale).")
        .example("$0 ranges", "List possible control ranges.")
        .example("$0 set absolute_zoom 200", "Zoom in.")
        .example("", "")
        .example(chalk_1.bold("Automate config:"), "")
        .example(chalk_1.dim("- Not all controls can be imported."), "")
        .example(chalk_1.dim("- Control order matters."), "")
        .example("$0 export > my-uvcc-export.json", "Save to file.")
        .example("cat my-uvcc-export.json | $0 import", "Load from file.")
        .example("", "")
        .example(chalk_1.bold("Target a specific device:"), "")
        .example(chalk_1.dim("- Only useful for multi-camera setups."), "")
        .example(chalk_1.dim("- For same-model cameras, also specify address."), "")
        .example(chalk_1.dim("- Alt. use system USB settings to find devices."), "")
        .example("$0 devices", "List available cameras.")
        .example("sudo $0 devices", "Avoid LIBUSB_ERROR_ACCESS.")
        .example("$0 --vendor 0x46d --product 0x82d export", "")
        .epilogue(epilogue);
    /* eslint-enable @typescript-eslint/prefer-readonly-parameter-types */
    return yargs_1.default.argv;
};
const mapArgv = (argv) => {
    // NOTE HACK: workaround yargs not being consistent with yargs.cmd versus yargs._ for defined/non-defined commands.
    // eslint-disable-next-line @typescript-eslint/dot-notation
    const cmd = typeof argv["cmd"] === "string"
        // eslint-disable-next-line @typescript-eslint/dot-notation
        ? argv["cmd"]
        : (typeof argv._[0] === "string"
            ? argv._[0]
            : null);
    const { address, control, product, value1, value2, vendor, verbose, } = argv;
    assert_1.default(typeof address === "number");
    assert_1.default(typeof cmd === "string");
    assert_1.default(typeof product === "number");
    assert_1.default(typeof vendor === "number");
    assert_1.default(typeof verbose === "boolean");
    // NOTE: accept one value if the first is a number, two if both are numbers.
    let values = [];
    if (typeof value1 === "number") {
        values = typeof value2 === "number" ? [
            value1,
            value2,
        ] : [
            value1,
        ];
    }
    const mappedArgv = {
        address,
        cmd,
        control: typeof control === "string" ? control : undefined,
        product,
        values,
        vendor,
        verbose,
    };
    return mappedArgv;
};
function runtimeConfigurator() {
    const rawArgv = getYargsArgv();
    const argv = mapArgv(rawArgv);
    return argv;
}
exports.default = runtimeConfigurator;
